@@ -39,7 +39,9 @@ class AutoSlugField(SlugField):
     (`unique_with`) or globally (`unique`) and adding a number to the slug to make
     it unique.
 
-    :param populate_from: string: name of attribute from which to fill the slug.
+    :param populate_from: string or callable: if string is given, it is considered
+        as the name of attribute from which to fill the slug. If callable is given,
+        it should accept `instance` parameter and return a value to fill the slug with.
     :param unique: boolean: ensure total slug uniqueness (unless more precise `unique_with`
         is defined).
     :param unique_with: string or tuple of strings: name or names of attributes
@@ -80,7 +82,7 @@ class AutoSlugField(SlugField):
         # globally unique, silently fix on conflict ("foo" --> "foo-1".."foo-n")
         slug = AutoSlugField(unique=True)
 
-        # autoslugify value from title attr; default editable to False
+        # autoslugify value from attribute named "title"; editable defaults to False
         slug = AutoSlugField(populate_from='title')
 
         # same as above but force editable=True
@@ -97,6 +99,13 @@ class AutoSlugField(SlugField):
 
         # minimum date granularity is shifted from day to month
         slug = AutoSlugField(populate_from='title', unique_with='pub_date__month')
+
+        # autoslugify value from a dynamic attribute (i.e. a method)
+        slug = AutoSlugField(populate_from='get_full_name')
+
+        # autoslugify value from a custom callable
+        # (ex. usage: user profile models)
+        slug = AutoSlugField(populate_from=lambda instance: instance.user.get_full_name())
     """
     def __init__(self, *args, **kwargs):
         kwargs['max_length'] = kwargs.get('max_length', 50)
@@ -130,17 +139,16 @@ class AutoSlugField(SlugField):
         # get currently entered slug
         value = self.value_from_object(instance)
 
-        slug = None
-
         # autopopulate (unless the field is editable and has some value)
-        if value:
-            slug = slugify(value)
-        elif self.populate_from: # and not self.editable:
-            slug = slugify(getattr(instance, self.populate_from))
-            if __debug__ and not slug:
-                print 'Failed to populate slug %s.%s from an empty field %s' % \
+        if self.populate_from and not value: # and not self.editable:
+            value = self._get_prepopulated_value(instance)
+
+            if __debug__ and not value:
+                print 'Failed to populate slug %s.%s from %s' % \
                     (instance._meta.object_name, self.name, self.populate_from)
 
+        slug = slugify(value)
+
         if not slug:
             # no incoming value,  use model name
             slug = instance._meta.module_name
@@ -158,6 +166,16 @@ class AutoSlugField(SlugField):
         return slug
         #return super(AutoSlugField, self).pre_save(instance, add)
 
+    def _get_prepopulated_value(self, instance):
+        """Returns preliminary value based on `populate_from`."""
+        if callable(self.populate_from):
+            # AutoSlugField(populate_from=lambda instance: ...)
+            return self.populate_from(instance)
+        else:
+            # AutoSlugField(populate_from='foo')
+            attr = getattr(instance, self.populate_from)
+            return callable(attr) and attr() or attr
+
     def _generate_unique_slug(self, instance, slug):
         """
         Generates unique slug by adding a number to given value until no model
@@ -235,7 +253,7 @@ class AutoSlugField(SlugField):
                 # raise model.DoesNotExist if current slug is unique
                 rival = model.objects.get(**dict(lookups + ((self.name, slug),) ))
                 # not unique, but maybe the "rival" is the instance itself?
-                if rival.id == instance.id:
+                if rival == instance:
                     raise model.DoesNotExist
                 # the slug is not unique; change once more
                 index += 1
